using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Forms;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace DynamicControlsExample
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("5C4F1D73-08F9-4A8E-B6BA-91D20D7FAA13"), ProgId("DynamicControlsExample.AddinModule"), RunInstaller(true)]
    public class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        public AddinModule()
        {
            InitializeComponent();
        }

        private AddinExpress.MSO.ADXOutlookAppEvents adxOutlookEvents;
 
        #region Component Designer generated code
        /// <summary>
        /// Required by designer
        /// </summary>
        private System.ComponentModel.IContainer components;
 
        /// <summary>
        /// Required by designer support - do not modify
        /// the following method
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.adxOutlookEvents = new AddinExpress.MSO.ADXOutlookAppEvents(this.components);
            // 
            // adxOutlookEvents
            // 
            this.adxOutlookEvents.NewInspector += new AddinExpress.MSO.ADXOlInspector_EventHandler(this.adxOutlookEvents_NewInspector);
            this.adxOutlookEvents.InspectorActivate += new AddinExpress.MSO.ADXOlInspector_EventHandler(this.adxOutlookEvents_InspectorActivate);
            // 
            // AddinModule
            // 
            this.AddinName = "DynamicControlsExample";
            this.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook;
            this.AddinBeginShutdown += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinBeginShutdown);
            this.AddinStartupComplete += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinStartupComplete);

		}
        #endregion
 
        #region ADX automatic code
 
        // Required by Add-in Express - do not modify
        // the methods within this region
 
        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }
 
        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }
 
        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }
 
        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion


		private AddinExpress.MSO.ADXCommandBar myBar;

		private AddinExpress.MSO.ADXCommandBarControl AddControl(object container, Type ctrlType, string caption, 
			string controlTag, int before, bool beginGroup, object eventHandler, bool temporary)
		{
			AddinExpress.MSO.ADXCommandBarControl adxNewControl = null;
			AddinExpress.MSO.ADXBaseControlCollection collection = null;
			
			if (container is AddinExpress.MSO.ADXCommandBar)
				collection = (container as AddinExpress.MSO.ADXCommandBar).Controls;
			else if (container is AddinExpress.MSO.ADXCommandBarPopup)
				collection = (container as AddinExpress.MSO.ADXCommandBarPopup).Controls;
			
			if (collection == null) return null;

			if (eventHandler != null)
			{
				adxNewControl = collection.GetControlByControlTag(controlTag, false);
				if (adxNewControl != null)
				{
					switch(ctrlType.Name)
					{
						case "ADXCommandBarButton":
							adxNewControl.AsButton.Click -= (eventHandler as AddinExpress.MSO.ADXClick_EventHandler);
							break;
						case "ADXCommandBarComboBox":
							adxNewControl.AsComboBox.Change -= (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
							break;
						case "ADXCommandBarDropDownList":
							adxNewControl.AsDropDownList.Change -= (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
							break;
						case "ADXCommandBarEdit":
							adxNewControl.AsEdit.Change -= (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
							break;
					}
				}
			}

			adxNewControl = collection.Add(ctrlType, controlTag, 1, before, temporary);
			adxNewControl.BeginGroup = beginGroup;
			adxNewControl.Caption = caption;
			switch(ctrlType.Name)
			{
				case "ADXCommandBarButton":
					adxNewControl.AsButton.Style = AddinExpress.MSO.ADXMsoButtonStyle.adxMsoButtonCaption;
					if (eventHandler != null)
						adxNewControl.AsButton.Click += (eventHandler as AddinExpress.MSO.ADXClick_EventHandler);
					break;
				case "ADXCommandBarComboBox":
					if (eventHandler != null)
						adxNewControl.AsComboBox.Change += (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
					break;
				case "ADXCommandBarDropDownList":
					if (eventHandler != null)
						adxNewControl.AsDropDownList.Change += (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
					break;
				case "ADXCommandBarEdit":
					if (eventHandler != null)
						adxNewControl.AsEdit.Change += (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
					break;
				case "ADXCommandBarPopup":
					break;
			}
			return adxNewControl;
		}

		private void AddinModule_AddinStartupComplete(object sender, System.EventArgs e)
		{
		}

		//private bool deleteWhenHostClose = true;

		private void AddinModule_AddinBeginShutdown(object sender, System.EventArgs e)
		{
			/*if (deleteWhenHostClose)
			{
				while(myBar.Controls.Count>0)
					myBar.Controls.RemoveAt(0);
				myBar.Dispose();
			}*/
		}


        private void DoClick(object sender)
        {
            MessageBox.Show((sender as AddinExpress.MSO.ADXCommandBarControl).Caption, AddinName);
        }

        private Outlook._Application OutlookApp
        {
            get
            {
                return HostApplication as Outlook._Application;
            }
        }

        private void DoClear(object sender)
        {
            AddinExpress.MSO.ADXCommandBarPopup popup =
                myBar.Controls.GetControlByControlTag("MY_UNIQUE_TAG 3", false) as AddinExpress.MSO.ADXCommandBarPopup;
            if (popup != null)
            {
                Outlook.Inspector inspector;
                Outlook.Inspectors collection = null;
                Microsoft.Office.Core.CommandBarPopup popupObj;
                try
                {
                    // clear the popup in the active inspector window.
                    popup.Controls.Clear();

                    collection = OutlookApp.Inspectors;
                    for (int i = 1; i <= collection.Count; i++)
                    {
                        inspector = null; popupObj = null;
                        try
                        {
                            inspector = collection[i];
                            this.CommandBarsObj = inspector.CommandBars;
                            popupObj = this.FindControlObj(Microsoft.Office.Core.MsoControlType.msoControlPopup, 
                                "MY_UNIQUE_TAG 3") as Microsoft.Office.Core.CommandBarPopup;
                            if (popupObj != null)
                            {
                                while (popupObj.Controls.Count > 0)
                                    popupObj.Controls[1].Delete(false);
                            }
                        }
                        finally
                        {
                            this.CommandBarsObj = null;
                            if (popupObj != null)
                                Marshal.ReleaseComObject(popupObj);
                            if (inspector != null)
                                Marshal.ReleaseComObject(inspector);
                        }
                    }
                }
                finally
                {
                    if (collection != null)
                        Marshal.ReleaseComObject(collection);
                }
            }
        }

		private void DoChange(object sender)
		{
			MessageBox.Show("The Text has been changed.", AddinName);
		}

        private void adxOutlookEvents_NewInspector(object sender, object inspector, string folderName)
        {
            Outlook._Inspector inspObj = 
                inspector as Outlook._Inspector;
            if (inspObj.CurrentItem is Outlook._MailItem)
            {
                this.CommandBarsObj = inspObj.CommandBars;
                // add a new bar or connect to the existing one
                myBar = this.CommandBarAdd(typeof(AddinExpress.MSO.ADXOlInspectorCommandBar), "My Dynamic Bar",
                    AddinExpress.MSO.ADXMsoBarPosition.adxMsoBarTop, true);
                if (myBar != null)
                {
                    AddinExpress.MSO.ADXCommandBarControl ctrl;

                    AddControl(myBar, typeof(AddinExpress.MSO.ADXCommandBarButton),
                        "My Action", "MY_UNIQUE_TAG 1", 0, true, new AddinExpress.MSO.ADXClick_EventHandler(DoClick), true);
                    AddControl(myBar, typeof(AddinExpress.MSO.ADXCommandBarEdit),
                        "", "MY_UNIQUE_TAG 2", 0, true, new AddinExpress.MSO.ADXChange_EventHandler(DoChange), true);

                    ctrl = AddControl(myBar, typeof(AddinExpress.MSO.ADXCommandBarPopup),
                        "My Popup Menu", "MY_UNIQUE_TAG 3", 0, false, null, true);
                    AddControl(ctrl, typeof(AddinExpress.MSO.ADXCommandBarButton),
                        "Popup Item 1", "MY_UNIQUE_TAG 4", 0, false, new AddinExpress.MSO.ADXClick_EventHandler(DoClick), true);
                    AddControl(ctrl, typeof(AddinExpress.MSO.ADXCommandBarButton),
                        "Popup Item 2", "MY_UNIQUE_TAG 5", 0, false, new AddinExpress.MSO.ADXClick_EventHandler(DoClick), true);
                    AddControl(ctrl, typeof(AddinExpress.MSO.ADXCommandBarButton),
                        "Popup Item 3", "MY_UNIQUE_TAG 6", 0, false, new AddinExpress.MSO.ADXClick_EventHandler(DoClick), true);

                    AddControl(myBar, typeof(AddinExpress.MSO.ADXCommandBarButton),
                        "Clear Popup", "MY_UNIQUE_TAG 7", 0, true, new AddinExpress.MSO.ADXClick_EventHandler(DoClear), true);
                    myBar.Visible = true;
                }
            }
        }

        private void adxOutlookEvents_InspectorActivate(object sender, object inspector, string folderName)
        {
            //
        }
	}
}

